/*************************************************************
 * lcd_main.c
 *
 * LCD main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <string.h>
#include <s1c176xx.h>

#ifdef	PE_LCD
#include <lcd_api.h>
#include <osc_api.h>


#if !defined(TRUE) || !defined(FALSE)
#define TRUE 1
#define FALSE 0
#endif

#define EXIT_LCD -1

/* comands */
#define LCD_ON_CMD			"on"
#define LCD_OFF_CMD			"off"
#define LCD_SEGON_CMD		"segon"
#define LCD_SEGOFF_CMD		"segoff"
#define LCD_MAIN_END_CMD	"exit"
#define LCD_TBL_END			"error"


#define RESET_INPUT_SEG		LCD8_SEG_MAX	//seg reset data
#define RESET_INPUT_COM		LCD8_COM_MAX	//com riset data


void lcdMain(void);
static short checkUserInput(const unsigned char* inputString);
static void startLCD(void);
static short onAllLCD(void);
static short offAllLCD(void);
static short segOnLCD(void);
static short segOffLCD(void);
static short checkInputArg(void);
static short exitLCD(void);


struct tag_LCD_CMD_ARGUMENT;
typedef struct tag_LCD_CMD_ARGUMENT {								// segon/segoff command
	unsigned short argSeg;												// segment
	unsigned short argCom;												// common
} LCD_CMD_ARGUMENT;					// argument for input commmand


static LCD_CMD_ARGUMENT cmdArg = {
	RESET_INPUT_SEG,
	RESET_INPUT_COM
};


struct  tag_LCD_CMD_TBL;
typedef struct  tag_LCD_CMD_TBL {									// input command
	unsigned char	command[10];										// command string
	signed short	(*function)(void);									// command function
} LCD_CMD_TBL;							//command table


/*
 * LCD sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void lcdMain(void)
{
	unsigned char inputString[20];	// user input data buffer
	unsigned char bufInput[7];		// first six characters of input data and EOS
	signed short exitflg;			// exit flag

	printf("<<< LCD driver demonstration start >>>\n");

	// start LCD clock
	CLG_OSC_setLCDClock(CLG_OSC_LCDCLK_HSCLK, CLG_OSC_LCDRATIO_64);
	CLG_OSC_controlLCDClock(CLG_OSC_LCDCLK_ENABLE);

	// start LCD
	startLCD();

	// wait recieve user input.
	while(1) {
		// input buffer reset
		memset(bufInput, '\0', sizeof(bufInput));
		cmdArg.argSeg = RESET_INPUT_SEG;
		cmdArg.argCom = RESET_INPUT_COM;

		// user input
		fgets((char*)&inputString[0], sizeof(inputString), stdin);
		sscanf((char*)&inputString[0], "%6s %x,%x", &bufInput[0], &cmdArg.argSeg, &cmdArg.argCom);

		// check user input
		exitflg = checkUserInput(bufInput);

		//exit
		if(exitflg == EXIT_LCD) {
			break;
		}
	}

	// input buffer clear
	sscanf((char*)&inputString[0], "%6s %x,%x", &bufInput[0], &cmdArg.argSeg, &cmdArg.argCom);

	// stop LCD
	LCD8_controlDisplay(LCD8_DISPLAY_OFF);

	// stop LCD clock
	CLG_OSC_controlLCDClock(CLG_OSC_LCDCLK_DISABLE);

	printf("<<< LCD driver demonstration finish >>>\n");
}

/*
 * set check user input strings.
 * param	user input strings
 * return	short
 * history	:	2009/12/25
 */
static short checkUserInput(const unsigned char* inputString)
{
	unsigned char	i = 0;		//loop counter
	signed short ret;			//return value

	LCD_CMD_TBL LCDcommandTable[] = {							// command table
		{ LCD_ON_CMD			, onAllLCD	},					//	on command
		{ LCD_OFF_CMD			, offAllLCD	},					//	off command
		{ LCD_SEGON_CMD			, segOnLCD	},					//	segon command
		{ LCD_SEGOFF_CMD		, segOffLCD	},					//	segoff command
		{ LCD_MAIN_END_CMD		, exitLCD	},					//	exit command
		{ LCD_TBL_END			, NULL		}					// table end
	};

	 // check command
	while(1) {
		// check table end
		if(strncmp((char*)LCDcommandTable[i].command, LCD_TBL_END, sizeof(LCD_TBL_END)) == 0) {
			// invalid input command
			ret = FALSE;
			break;
		}

		// check input command
		if(strncmp((char*)LCDcommandTable[i].command, (char*)inputString, sizeof(LCDcommandTable[i].command)) == 0) {
			// run function
			ret = LCDcommandTable[i].function();
			break;
		}

		++i;
	}
	return ret;
}


/*
 * Start LCD.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startLCD(void)
{
	LCD8_initPower(LCD8_PROTECTMODE_OFF, LCD8_POWERSELECT_1V);
	LCD8_init(LCD8_CONTRAST_7, SET_LCD_DUTY);
	LCD8_controlDisplay(LCD8_DISPLAY_NORMAL);
}

/*
 * Turn on the all LCD.
 * param	void
 * return	short									end status
 * history	:	2009/12/25
 */
static short onAllLCD(void)
{
	// all on
	LCD8_controlDisplay(LCD8_DISPLAY_ALLON);

	return TRUE;
}

/*
 * Turn off the all LCD light.
 * param	void
 * return	short									end status
 * history	:	2009/12/25
 */
static short offAllLCD(void)
{
	// all off
	LCD8_controlDisplay(LCD8_DISPLAY_ALLOFF);

	return TRUE;
}


/*
 * Turn on the selected segment.
 * param	void
 * return	short									end status
 * history	:	2009/12/25
 */
static short segOnLCD(void)
{
	signed short checkflg;	// check input flag

	// check input arguments
	checkflg = checkInputArg();

	if(checkflg == TRUE) {												// input OK
		// set normal display
		LCD8_controlDisplay(LCD8_DISPLAY_NORMAL);

		// 1-segment on
		LCD8_display1Seg((unsigned char)(LCD8_SEG00 + cmdArg.argSeg), (unsigned char)(LCD8_COM00 + cmdArg.argCom), LCD8_DOT_H);
	}

	return TRUE;
}

/*
 * Turn off the selected segment.
 * param	void
 * return	short									end status
 * history	:	2009/12/25
 */
static short segOffLCD(void)
{
	signed short checkflg;	// check input flag

	// check input arguments
	checkflg = checkInputArg();

	if(checkflg == TRUE) {												// input OK
		// seg normal display
		LCD8_controlDisplay(LCD8_DISPLAY_NORMAL);

		// 1-segment off
		LCD8_display1Seg((unsigned char)(LCD8_SEG00 + cmdArg.argSeg), (unsigned char)(LCD8_COM00 + cmdArg.argCom), LCD8_DOT_L);
	}

	return TRUE;
}


/*
 * Check user input arguments.
 * param	void
 * return	short									result
 * history	:	2009/12/25
 */
static short checkInputArg(void)
{
	signed short ret;			//return value

	// arguments range check
	if((cmdArg.argSeg >= LCD8_SEG_MAX) || (cmdArg.argCom >= LCD8_COM_MAX)) {
		ret = FALSE;													// range over
	} else {
		ret = TRUE;														// range check OK
	}

	return ret;
}


/*
 * Retun exit order.
 * param	void
 * return	short									end status
 * history	:	2009/12/25
 */
static short exitLCD(void)
{
	// LCD sample program exit
	return EXIT_LCD;
}


#endif	// end of PE_LCD
