/*************************************************************
 * spim_main.c
 *
 * SPI master main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_SPI_MASTER
#include <string.h>

#include <mux_api.h>
#include <port_api.h>
#include <spi_api.h>
#include <t16_api.h>
#include "spi_main.h"

// send data number
#define SPI_MST_MAIN_SENDDATA		"FROM MST"
#define SPI_MST_MAIN_SENDDATA_NUM	8
#define SPI_MST_MAIN_RECVDATA_NUM	8

#define SPI_MST_MAIN_CH				SPI_CH0		// SPI ch

#define SPI_MST_MAIN_CS_NEGATE		0xFF		// SS negate
#define SPI_MST_MAIN_CS_ASSERT		0xBB		// SS assert

#define	SPI_10KHZ					10000		// for SPI clock

/* data variable */
static volatile  SPI_MAIN_PARAM paramSPI;


// prototype
void spiMasterMain(void);
static void startT16(unsigned char chNo);
static void setPortOutput(void);
static void clearPortOutput(void);
static void startSPIMaster(unsigned char chNo);
static void stopSPIMaster(unsigned char chNo);
unsigned char sendSPIData(unsigned char chNo, unsigned char *buf, unsigned char len);
unsigned char recvSPIData(unsigned char chNo, unsigned char *buf, unsigned char len);
void spiIntHandler(void)__attribute__((interrupt_handler));


/*
 * SPI master sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void spiMasterMain(void)
{
	/* send data variable */
	unsigned char	spiMasterSendData[] = SPI_MST_MAIN_SENDDATA;	// send data charactors
	unsigned char	data[SPI_MST_MAIN_RECVDATA_NUM + 1];			// receive length
	unsigned char	waitPressEnter[2];								// key input buffer

	printf("<<< SPI master demonstration start >>>\n");

	memset(data, '\0', sizeof(data));

	// start 16 bit timer channel 1
	startT16(T16_CH1);

	// set ports to a output mode
	setPortOutput();

	// start SPI master
	startSPIMaster(SPI_MST_MAIN_CH);

	printf("Transmitted data: ");

	// send data to SPI slave
	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_ASSERT);

	sendSPIData(SPI_MST_MAIN_CH, &spiMasterSendData[0], (unsigned char)strlen((char*)&spiMasterSendData[0]));
	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_NEGATE);

	printf("Please press enter key\n");
	fgets((char*)&waitPressEnter[0], 2, stdin);

	printf("Received data: ");

	// receive data from SPI slave
	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_ASSERT);
	recvSPIData(SPI_MST_MAIN_CH, &data[0], SPI_MST_MAIN_RECVDATA_NUM);
	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_NEGATE);

	// stop SPI master
	stopSPIMaster(SPI_MST_MAIN_CH);

	// clear ports to a output mode
	clearPortOutput();

	// stop T16 timer
	T16_setTimerRun(T16_CH1, T16_TIMER_STOP);

	// clear port
	MUX_setSPIport(SPI_MST_MAIN_CH, MUX_OFF);

	printf("<<< SPI master demonstration finish >>>\n");
}

/*
 * start 16 bit timer.
 * param	unsigned char  chNo      T16 channel number.
 * return	void
 * history	:	2009/12/25
 */

static void startT16(unsigned char chNo)
{
	T16_TIM_MODE	mode;	// timer mode setting
	unsigned char	tr;

	mode.ckactv	= T16_TIMMODE_ACTVH;
	mode.cksl	= T16_TIMMODE_INTERNAL;
	mode.trmd	= T16_TIMMODE_REPEAT;
	tr = ((OSC3_CLK / 4) / (SPI_10KHZ * 2)) - 1;
	T16_init(chNo, T16_CNTCLK_4, &mode, tr);
	T16_setTimerRun(chNo, T16_TIMER_RUN);
}


/*
 * set registers for outputs.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void setPortOutput(void)
{
	// set output settings
	PORT_controlOutput(PORT_NO_P1, PORT_PX2_ENABLE);

	PORT_controlInput(PORT_NO_P1, PORT_PX2_DISABLE);

	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_NEGATE);
}

/*
 * clear registers for outputs.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void clearPortOutput(void)
{
	// clear output settings
	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_NEGATE);

	PORT_controlOutput(PORT_NO_P1, PORT_PX2_DISABLE);

	PORT_controlInput(PORT_NO_P1, PORT_PX2_DISABLE );

	PORT_setOutputData(PORT_NO_P1, SPI_MST_MAIN_CS_NEGATE);

}


/*
 * start SPI master.
 * param	SPI master channel number.
 * return	void
 * history	:	2009/12/25
 */
static void startSPIMaster(unsigned char chNo)
{
	SPI_INIT_PARAM		param;											// initialize parameter

	param.selectClk			= SPI_CLK_T16;
	param.selectMLSB		= SPI_MSB_FIRST;
	param.selectPhase		= SPI_PHASE_DATAIN;
	param.selectPolarity	= SPI_ACTIVE_L;

	param.selectMode		= SPI_MODE_MASTER;
	SPI_init(chNo, &param);

	SPI_initInt(chNo, SPI_INT_LEVEL3);
	SPI_setComEnable(chNo, SPI_CTL_ENABLE);
}

/*
 * stop SPI master.
 * param	SPI master channel number.
 * return	void
 * history	:	2009/12/25
 */
static void stopSPIMaster(unsigned char chNo)
{
	SPI_setComEnable(chNo, SPI_CTL_DISABLE);
	SPI_initInt(chNo, SPI_INT_LEVEL0);
}

/*
 * start SPI data send.
 * param	unsigned char  chNo      SPI channel number.
 * param	unsigned char  *buf      send data
 * param	unsigned char  len       send data length
 * return	unsigned char            actual send data length
 * history	:	2009/12/25
 */
unsigned char sendSPIData(unsigned char chNo, unsigned char *buf, unsigned char len)
{
	unsigned char	actual;			// actual send data length

	actual = 0;
	paramSPI.dataTx = buf;
	paramSPI.counterTx = len;

	// enable interrupt
	SPI_controlInt(chNo, (SPI_RXINT_DISABLE | SPI_TXINT_ENABLE));

	// wait all data is sent
	while(paramSPI.counterTx > 0) {
		asm("halt");
		asm("nop");
	}

	// disable interrupt
	SPI_controlInt(chNo, (SPI_RXINT_DISABLE | SPI_TXINT_DISABLE));

	actual = (unsigned char)(len - paramSPI.counterTx);
	printf("%s\n", (paramSPI.dataTx - actual));

	return actual;
}

/*
 * start SPI data send.
 * param	unsigned char  chNo      SPI channel number.
 * param	unsigned char  *buf      receive data
 * param	unsigned char  len       receive data length
 * return	unsigned char            actual receive data length
 * history	:	2009/12/25
 */
unsigned char recvSPIData(unsigned char chNo, unsigned char *buf, unsigned char len)
{
	unsigned char	actual;			// actual receive data length

	actual = 0;
	paramSPI.dataRx = buf;
	paramSPI.counterRx = 0;
	paramSPI.counterMaxRx = len;

	SPI_getReceiveData(chNo, &paramSPI.dataRx[0]);

	// enable interrupt
	SPI_controlInt(chNo, (SPI_RXINT_ENABLE | SPI_TXINT_DISABLE));
	SPI_setTransmitData(chNo, 0x00);

	// wait all data is sent
	while(paramSPI.counterRx < paramSPI.counterMaxRx) {
		asm("halt");
		asm("nop");
	}

	// disable interrupt
	SPI_controlInt(chNo, (SPI_RXINT_DISABLE | SPI_TXINT_DISABLE));

	actual = paramSPI.counterRx;
 	*(paramSPI.dataRx) = '\0';
	printf("%s\n", (paramSPI.dataRx - actual) );

	return actual;
}

/*
 * SPI master/slave interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void spiIntHandler(void)
{
	unsigned short	flgIntr;											// interrupt flag

	// get interrupt flag
	SPI_checkIntFlag(SPI_MST_MAIN_CH, &flgIntr);

	// transmit
	if((flgIntr & SPI_TXFLAG_EMPTY) == SPI_TXFLAG_EMPTY) {		//SPI_TXFLAG_EMPTY = 0x01
		if(paramSPI.counterTx > 0) {
			// set data
			SPI_setTransmitData(SPI_MST_MAIN_CH, *paramSPI.dataTx);
			paramSPI.counterTx--;
			paramSPI.dataTx++;
		}
	}

	// receive
	if((flgIntr & SPI_RXFLAG_FULL) == SPI_RXFLAG_FULL) {		//SPI_RXFLAG_FULL = 0x02
		if(paramSPI.counterRx < paramSPI.counterMaxRx) {
			// get data
			SPI_getReceiveData(SPI_MST_MAIN_CH, paramSPI.dataRx);
			paramSPI.counterRx++;
			paramSPI.dataRx++;
			// set dummy data to receive next data
			if(paramSPI.counterRx != paramSPI.counterMaxRx) {
				SPI_setTransmitData(SPI_MST_MAIN_CH, 0x00);
			}
		}
	}
}
#endif	// end of PE_SPI_MASTER
