/**
 * @file	adc.c
 * @brief	ADC driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_ADC
#include <adc_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short ADC_init(ADC_INIT_PARAM *param);
short ADC_setChannel(unsigned short startCh, unsigned short endCh);
short ADC_setStoreMode(unsigned short selectBit);
short ADC_setConversionMode(unsigned short selectBit);
short ADC_setConversionTrigger(unsigned short selectBit);
short ADC_setSamplingClock(unsigned short selectBit);
short ADC_setDividedFrequency(unsigned short selectBit);
short ADC_setEnable(unsigned short selectBit);
void ADC_getResult(unsigned short *result);
void ADC_getConversionChannel(unsigned short *result);
void ADC_checkBusyStatus(unsigned short *result);
short ADC_controlTrigger(unsigned short selectBit);
short ADC_initInt(unsigned char intLevel);
short ADC_controlInt(unsigned short selectBit);
void ADC_resetIntFlag(void);
void ADC_checkIntFlag(unsigned short *flag);


/**
 * @brief	ADC initialize.
 * - 1. check parameters.
 * - 2. stop an A/D converter.
 * - 3. set A/D converter ports.
 * - 4. set an A/D converter clock divided frequency.
 * - 5. set an end channel.
 * - 6. set a start channel.
 * - 7. set a conversion mode.
 * - 8. set a conversion trigger.
 * - 9. set a sampling clock for analog input.
 * - 10. set a converted data store mode.
 *
 * @param	*param; ADC_INIT_PARAM:			ADC initialize parameter
 * -		endCh; unsigned short:			select end channel
 * 	-			refer to ADC_setChannel
 * -		startCh; unsigned short:		select start channel
 * 	-			refer to ADC_setChannel
 * -		storeMode; unsigned short:		select converted data store mode
 * 	-			refer to ADC_setStoreMode
 * -		convMode; unsigned short:		select conversion mode
 * 	-			refer to ADC_setConversionMode
 * -		convTrig; unsigned short:		select conversion trigger
 * 	-			refer to ADC_setConversionTrigger
 * -		smpClock; unsigned short:		select sampling clock for analog input
 * 	-			refer to ADC_setSamplingClock
 * -		convClock; unsigned short:		select A/D converter clock divided frequency
 * 	-			refer to ADC_setDividedFrequency
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_init(ADC_INIT_PARAM *param)
{
	unsigned char	use;													// #ADTRG use / no use
	
	// parameter check
	if(	(param->endCh >= ADC_CH_MAX) ||
		(param->startCh >= ADC_CH_MAX) ||
		(param->storeMode > ADC_STORE_MODE1) ||
		(param->convMode > ADC_CONVMODE_CONTINUOUS) ||
		(param->convTrig >= ADC_CONVTRIG_MAX) ||
		(param->convTrig == ADC_CONVTRIG_RESERVED) ||
		(param->smpClock >= ADC_SAMPLING_MAX) ||
		(param->convClock >= ADC_CONVCLK_MAX) ) {
			return	ADC_INVALID_PARAMETER;
	}
	
	// A/D conversion disable
	ADC_setEnable(ADC_DISABLE);
	
	// A/D port set
	if(param->convTrig == ADC_CONVTRIG_ADTRG) {
		use = MUX_ADTRG_ON;
	} else {
		use = MUX_ADTRG_OFF;
	}
	MUX_setADCport(use, param->startCh, param->endCh, MUX_ON);
	
	// A/D converter clock divided frequency set
	ADC_setDividedFrequency(param->convClock);
	
	// A/D conversion channel set
	ADC_setChannel(param->startCh, param->endCh);
	
	// conversion mode set
	ADC_setConversionMode(param->convMode);
	
	// conversion trigger set
	ADC_setConversionTrigger(param->convTrig);
	
	// sampling clock count set
	ADC_setSamplingClock(param->smpClock);
	
	// converted data store mode set
	ADC_setStoreMode(param->storeMode);
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set an A/D conversion channel.
 * - 1. check parameters.
 * - 2. set an A/D conversion start channel number.
 * - 3. set an A/D conversion end channel number.
 *
 * @param	startCh; unsigned short: 		select start channel
 * -		ADC_CH0 : conversion start channel number 0
 * -		ADC_CH1 : conversion start channel number 1
 * -		ADC_CH2 : conversion start channel number 2
 * -		ADC_CH3 : conversion start channel number 3
 * -		ADC_CH4 : conversion start channel number 4
 * -		ADC_CH5 : conversion start channel number 5
 * -		ADC_CH6 : conversion start channel number 6
 * -		ADC_CH7 : conversion start channel number 7
 * @param	endCh; unsigned short:			select end channel
 * -		ADC_CH0 : conversion end channel number 0
 * -		ADC_CH1 : conversion end channel number 1
 * -		ADC_CH2 : conversion end channel number 2
 * -		ADC_CH3 : conversion end channel number 3
 * -		ADC_CH4 : conversion end channel number 4
 * -		ADC_CH5 : conversion end channel number 5
 * -		ADC_CH6 : conversion end channel number 6
 * -		ADC_CH7 : conversion end channel number 7
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setChannel(unsigned short startCh, unsigned short endCh)
{
	// parameter check
	if(	(startCh >= ADC_CH_MAX) ||
		(endCh >= ADC_CH_MAX) ) {
			return	ADC_INVALID_PARAMETER;
	}
	
	// A/D conversion channel set
	RegModify(R16_ADC_TRG, (ADCE | ADCS), ((endCh << ADCE_SFT) | (startCh << ADCS_SFT)));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set a converted data store mode.
 * - 1. check parameters.
 * - 2. set a converted data store mode.
 *
 * @param	selectBit; unsigned short: 		select store mode
 * -		ADC_STORE_MODE0 : ADD[15:10]=0, ADD[9]=MSB, ADD[0]=LSB
 * -		ADC_STORE_MODE1 : ADD[15]=MSB,  ADD[6]=LSB, ADD[5:0]=0
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setStoreMode(unsigned short selectBit)
{
	// parameter check
	if(selectBit > ADC_STORE_MODE1) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// store mode set
	RegModify(R16_ADC_TRG, STMD, (selectBit << STMD_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set a conversion mode.
 * - 1. check parameters.
 * - 2. set an A/D conversion mode.
 *
 * @param	selectBit; unsigned short: 		select conversion mode
 * -		ADC_CONVMODE_SINGLE     : single conversion mode
 * -		ADC_CONVMODE_CONTINUOUS : continuous conversion mode
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setConversionMode(unsigned short selectBit)
{
	// parameter check
	if(selectBit > ADC_CONVMODE_CONTINUOUS) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// conversion mode set
	RegModify(R16_ADC_TRG, ADMS, (selectBit << ADMS_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set a conversion trigger.
 * - 1. check parameters.
 * - 2. set an A/D conversion trigger.
 *
 * @param	selectBit; unsigned short: 		select conversion trigger
 * -		ADC_CONVTRIG_SOFT  : software
 * -		ADC_CONVTRIG_T16   : 16-bit timer ch0
 * -		ADC_CONVTRIG_ADTRG : external trigger(xADTRG pin)
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setConversionTrigger(unsigned short selectBit)
{
	// parameter check
	if(	(selectBit >= ADC_CONVTRIG_MAX) ||
		(selectBit == ADC_CONVTRIG_RESERVED) ) {
			return	ADC_INVALID_PARAMETER;
	}
	
	// conversion trigger set
	RegModify(R16_ADC_TRG, ADTS, (selectBit << ADTS_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set a sampling clock count.
 * - 1. check parameters.
 * - 2. set a sampling clock count.
 *
 * @param	selectBit; unsigned short: 		select sampling clock count
 * -		ADC_SAMPLING_CLOCK2 : 2 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK3 : 3 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK4 : 4 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK5 : 5 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK6 : 6 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK7 : 7 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK8 : 8 counts of sampling clock
 * -		ADC_SAMPLING_CLOCK9 : 9 counts of sampling clock
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setSamplingClock(unsigned short selectBit)
{
	// parameter check
	if(selectBit >= ADC_SAMPLING_MAX) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// conversion mode set
	RegModify(R16_ADC_TRG, ADST, (selectBit << ADST_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set an A/D converter clock divided frequency.
 * - 1. check parameters.
 * - 2. set an A/D converter clock divided frequency.
 *
 * @param	selectBit; unsigned short: 		select divided frequency
 * -		ADC_CONVCLK2     : PCLK 1/2
 * -		ADC_CONVCLK4     : PCLK 1/4
 * -		ADC_CONVCLK8     : PCLK 1/8
 * -		ADC_CONVCLK16    : PCLK 1/16
 * -		ADC_CONVCLK32    : PCLK 1/32
 * -		ADC_CONVCLK64    : PCLK 1/64
 * -		ADC_CONVCLK128   : PCLK 1/128
 * -		ADC_CONVCLK256   : PCLK 1/256
 * -		ADC_CONVCLK512   : PCLK 1/512
 * -		ADC_CONVCLK1024  : PCLK 1/1024
 * -		ADC_CONVCLK2048  : PCLK 1/2048
 * -		ADC_CONVCLK4096  : PCLK 1/4096
 * -		ADC_CONVCLK8192  : PCLK 1/8192
 * -		ADC_CONVCLK16384 : PCLK 1/16384
 * -		ADC_CONVCLK32768 : PCLK 1/32768
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setDividedFrequency(unsigned short selectBit)
{
	// parameter check
	if(selectBit >= ADC_CONVCLK_MAX) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// A/D converter clock divided frequency set
	RegWrite(R16_ADC_DIV, selectBit);
	
	return	ADC_SUCCESS;
}

/**
 * @brief	set an A/D conversion enable / disable.
 * - 1. check parameters.
 * - 2. set an A/D conversion enable / disable.
 *
 * @param	selectBit; unsigned short: 		select enable / disable
 * -		ADC_DISABLE : A/D conversion disable
 * -		ADC_ENABLE  : A/D conversion enable
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_setEnable(unsigned short selectBit)
{
	// parameter check
	if(selectBit > ADC_ENABLE) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// A/D enable / disablet
	RegModify(R16_ADC_CTL, ADEN, (selectBit << ADEN_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	get an A/D converted data.
 * - 1. get an A/D converted data and set to "result".
 *
 * @param	*result; unsigned short: 		converted data
 * -		from 0 to 1023
 * @return	none
 * @date	2009/12/25
 */
void ADC_getResult(unsigned short *result)
{
	// A/D converted data get
	*result = RegRead(R16_ADC_ADD);
}

/**
 * @brief	get an A/D conversion channel status.
 * - 1. get an A/D conversion channel number and set to "result".
 *
 * @param	*result; unsigned short: 		channel number
 * -		ADC_CH0 : A/D conversion channel number 0
 * -		ADC_CH1 : A/D conversion channel number 1
 * -		ADC_CH2 : A/D conversion channel number 2
 * -		ADC_CH3 : A/D conversion channel number 3
 * -		ADC_CH4 : A/D conversion channel number 4
 * -		ADC_CH5 : A/D conversion channel number 5
 * -		ADC_CH6 : A/D conversion channel number 6
 * -		ADC_CH7 : A/D conversion channel number 7
 * @return	none
 * @date	2009/12/25
 */
void ADC_getConversionChannel(unsigned short *result)
{
	// internal conversion channel status get
	*result = (RegRead(R16_ADC_CTL) & ADICH) >> ADICH_SFT;
}

/**
 * @brief	check an internal BUSY status.
 * - 1. get an internal BUSY status and set the status to "result".
 *
 * @param	*result; unsigned short: 		busy status
 * -		ADC_STATUS_IDLE : A/D conversion is idle
 * -		ADC_STATUS_BUSY : A/D conversion is busy
 * @return	none
 * @date	2009/12/25
 */
void ADC_checkBusyStatus(unsigned short *result)
{
	// internal BUSY status get
	*result = (RegRead(R16_ADC_CTL) & ADIBS) >> ADIBS_SFT;
}

/**
 * @brief	control a software trigger.
 * - 1. check parameters.
 * - 2. set a software trigger start/stop.
 *
 * @param	selectBit; unsigned short: 		select start / stop
 * -		ADC_CONTROL_STOP  : A/D conversion stop
 * -		ADC_CONTROL_START : software trigger
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_controlTrigger(unsigned short selectBit)
{
	// parameter check
	if(selectBit > ADC_CONTROL_START) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// control conversion
	RegModify(R16_ADC_CTL, ADCTL, (selectBit << ADCTL_SFT));
	
	return	ADC_SUCCESS;
}

/**
 * @brief	ADC interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		ADC_INT_LEVEL0 : level0
 * -		ADC_INT_LEVEL1 : level1
 * -		ADC_INT_LEVEL2 : level2
 * -		ADC_INT_LEVEL3 : level3
 * -		ADC_INT_LEVEL4 : level4
 * -		ADC_INT_LEVEL5 : level5
 * -		ADC_INT_LEVEL6 : level6
 * -		ADC_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= ADC_INT_LEVELMAX) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(ADC_INTLVL, intLevel);
	
	return	ADC_SUCCESS;
}

/**
 * @brief	ADC interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		ADC_OVWINT_DISABLE  : overwrite error interrupt disable
 * -		ADC_OVWINT_ENABLE   : overwrite error interrupt enable
 * -		ADC_COMPINT_DISABLE : conversion complete interrupt disable
 * -		ADC_COMPINT_ENABLE  : conversion complete interrupt enable
 * @return	short:							end status
 * -		ADC_SUCCESS : success
 * -		ADC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short ADC_controlInt(unsigned short selectBit)
{
	// parameter check
	if((selectBit & ~ADC_IEN_MASK) != 0x0000) {
		return	ADC_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	ADC_resetIntFlag();
	
	// set interrupt enable / disable
	RegModify(R16_ADC_CTL, ADC_IEN_MASK, selectBit);
	
	return	ADC_SUCCESS;
}

/**
 * @brief	ADC interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @return	none
 * @date	2009/12/25
 */
void ADC_resetIntFlag(void)
{
	// reset interrupt flag
	RegModify(R16_ADC_CTL, ADOWE, ADC_OVWINT_RESET);
}

/**
 * @brief	ADC interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned short:			check interrupt flag
 * -		ADC_OVWINT_NORMAL        : overwrite normal
 * -		ADC_OVWINT_ERROR         : overwrite error
 * -		ADC_COMPINT_NOTCOMPLETED : conversion is not complete
 * -		ADC_COMPINT_COMPLETED    : conversion is complete
 * @return	none
 * @date	2009/12/25
 */
void ADC_checkIntFlag(unsigned short *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_ADC_CTL) & (ADOWE | ADCF);
}

#endif	// end of PE_ADC
