/**
 * @file	rtc.c
 * @brief	real time clock driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_RTC
#include <rtc_api.h>

/*
 * definition of "define"
 */

/*
 * definition of constants
 */

/*
 * Declarations of function prototype
 */
short RTC_init(unsigned char rtc24h);
short RTC_setTimerRun(unsigned char selectBit);
short RTC_setTimerMode(unsigned char rtc24h);
short RTC_setTime(RTC_DATE_INFO *pDate);
void RTC_setAdj(void);
void RTC_getTime(RTC_DATE_INFO *pDate);
void RTC_checkBusy(void);
short RTC_initInt(unsigned char intLevel, unsigned char intCycle, unsigned char intMode);
short RTC_controlInt(unsigned char selectBit);
void RTC_resetIntFlag(void);
void RTC_checkIntFlag(unsigned char *flag);


/**
 * @brief	RTC initialize
 * - 1. check parameters.
 * - 2. stop RTC interrupt
 * - 3. reset RTC interrupt flag
 * - 4. software reset
 * - 5. disable read buffer
 * - 6. start RTC
 * - 7. execute busy check
 * - 8. stop RTC
 *
 * @param	rtc24h;	unsigned char: 		select 12H/24H mode
 * -		RTC_MODE_12H : 12H mode
 * -		RTC_MODE_24H : 24H mode
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_init(unsigned char rtc24h)
{
	// parameter check
	if(rtc24h > RTC_MODE_24H) {
		return	RTC_INVALID_PARAMETER;
	}

	// execute software reset
	RegModify(R08_RTC_CNTL0, RTCRST, (RTCRST * RTC_SWRESET));
	RegModify(R08_RTC_CNTL0, RTCRST, (RTCRST * RTC_RSTCANCEL));

	// disable RTC read buffer
	RegModify(R08_RTC_CNTL1, RTCRDHLD, (RTC_RBOFF << RTCRDHLD_SFT));

	// set RTC mode
	RTC_setTimerMode(rtc24h);

	// start RTC
	RTC_setTimerRun(RTC_TIMER_RUN);

	// execute RTC busy check
	RTC_checkBusy();

	// stop RTC
	RTC_setTimerRun(RTC_TIMER_STOP);

	return RTC_SUCCESS;
}

/**
 * @brief	set RTC run / stop.
 * - 1. check parameters.
 * - 2. set RTC run/stop.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		RTC_TIMER_RUN  : real time clock run
 * -		RTC_TIMER_STOP : real time clock stop
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_setTimerRun(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RTC_TIMER_STOP) {
		return	RTC_INVALID_PARAMETER;
	}

	// RTC run / stop
	RegModify(R08_RTC_CNTL0, RTCSTP, (selectBit << RTCSTP_SFT));

	return	RTC_SUCCESS;
}

/**
 * @brief	select mode 12H/24H.
 * - 1. check parameters.
 * - 2. stop RTC
 * - 2. set RTC mode.
 *
 * @param	rtc24h;	unsigned char: 		select 12H/24H mode
 * -		RTC_MODE_12H : 12H mode
 * -		RTC_MODE_24H : 24H mode
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_setTimerMode(unsigned char rtc24h)
{
	// parameter check
	if(rtc24h > RTC_MODE_24H) {
		return	RTC_INVALID_PARAMETER;
	}

	// stop RTC
	RTC_setTimerRun(RTC_TIMER_STOP);

	// set RTC timer mode
	RegModify(R08_RTC_CNTL0, RTC24H, (rtc24h << RTC24H_SFT));

	return RTC_SUCCESS;
}

/**
 * @brief	set RTC time.
 * - 1. check parameters.
 * - 2. execute busy check
 * - 3. set RTC time
 * - 4. set RTC counter hold control
 * - 5. start RTC
 *
 * @param	*pDate; RTC_DATE_INFO:			RTC time data
 * -		second; unsigned char:			second count data(0x00-0x59)
 * -		minute; unsigned char:			minute count data(0x00-0x59)
 * -		hour; unsigned char:			hour count data(0x00-0x01 or 0x00-0x02)
 * -		indicator; unsigned char:		indicate AM/PM
 * 	-			RTC_INDICATOR_AM  : AM
 * 	-			RTC_INDICATOR_PM  : PM
 * -		day; unsigned char:				day count data(0x01-0x31)
 * -		month; unsigned char:			month count data(0x01-0x12)
 * -		year; unsigned char:			year count data(0x00-0x99)
 * -		week; unsigned char:			week count data
 * 	-			RTC_WEEK_SUNDAY  : Sunday
 * 	-			RTC_WEEK_MONDAY  : Monday
 * 	-			RTC_WEEK_TUESDAY  : Tuesday
 * 	-			RTC_WEEK_WEDNESDAY  : Wednesday
 * 	-			RTC_WEEK_THURSDAY  : Thursday
 * 	-			RTC_WEEK_FRIDAY  : Friday
 * 	-			RTC_WEEK_SATURDAY  : Saturday
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_setTime(RTC_DATE_INFO *pDate)
{
	// parameter check
	if(	(((pDate->second) & RTCSL) > RTC_COUNT_MAX) ||
		((pDate->second) > RTC_SCOUNT_MAX) ||
		(((pDate->minute) & RTCMIL) > RTC_COUNT_MAX) ||
		((pDate->minute) > RTC_MICOUNT_MAX) ||
		(((pDate->hour) & RTCHL) > RTC_COUNT_MAX) ||
		((pDate->hour) > RTC_HCOUNT_MAX) ||
		((pDate->indicator) > RTC_INDICATOR_PM) ||
		(((pDate->day) & RTCDL) > RTC_COUNT_MAX) ||
		((pDate->day) < RTC_DCOUNT_MIN) ||
		((pDate->day) > RTC_DCOUNT_MAX) ||
		(((pDate->month) & RTCMOL) > RTC_COUNT_MAX) ||
		((pDate->month) < RTC_MOCOUNT_MIN) ||
		((pDate->month) > RTC_MOCOUNT_MAX) ||
		(((pDate->year) & RTCYL) > RTC_COUNT_MAX) ||
		((pDate->year) > RTC_YCOUNT_MAX) ||
		((pDate->week) > RTC_WEEK_SATURDAY) ) {
		return RTC_INVALID_PARAMETER;
	}

	// execute RTC busy check
	RTC_checkBusy();

	// set RTC time
	// set second data
	RegWrite(R08_RTC_SEC, (pDate->second));

	// set minute data
	RegWrite(R08_RTC_MIN, (pDate->minute));

	// set hour data
	RegWrite(R08_RTC_HOUR, (pDate->hour));

	if ((((RegRead(R08_RTC_CNTL0)) & RTC24H) >> RTC24H_SFT) == RTC_12H) {
		RegModify(R08_RTC_HOUR, RTCAP, ((pDate->indicator) << RTCAP_SFT));
	} else {
		RegModify(R08_RTC_HOUR, RTCAP, RTC_INDICATOR_AM);
	}

	// set day data
	RegWrite(R08_RTC_DAY, (pDate->day));

	// set month data
	RegWrite(R08_RTC_MONTH, (pDate->month));

	// set year data
	RegWrite(R08_RTC_YEAR, (pDate->year));

	// set week data
	RegWrite(R08_RTC_WEEK, (pDate->week));

	// set RTC counter hold register normal run
	RegModify(R08_RTC_CNTL1, RTCHLD, (RTC_COUNTER_RUNNING << RTCHLD_SFT));

	return RTC_SUCCESS;
}


/**
 * @brief	RTC 30-second adjust.
 * - 1. execute counter adjust.
 *
 * @return	none
 * @date	2010/04/01
 */
void RTC_setAdj(void)
{
	// execute timer adjust
	RegModify(R08_RTC_CNTL0, RTCADJ, (RTCADJ * RTC_ADJUST));
}

/**
 * @brief	set RTC time.
 * - 1. check parameters.
 * - 2. execute busy check
 * - 3. set RTC time
 * - 4. set RTC counter hold control
 * - 5. start RTC
 *
 * @param	*pDate; RTC_DATE_INFO:			RTC time data
 * -		refer to RTC_setTime
 * @return	none
 * @date	2010/03/29
 */
void RTC_getTime(RTC_DATE_INFO *pDate)
{
	// enable RTC read buffer
	RegModify(R08_RTC_CNTL1, RTCRDHLD, (RTC_RBON << RTCRDHLD_SFT));

	// read RTC counter
	// read second data
	pDate->second = RegRead(R08_RTC_SEC);

	// read minute data
	pDate->minute = RegRead(R08_RTC_MIN);

	// read hour data
	pDate->hour = (RegRead(R08_RTC_HOUR)) & (RTCHH | RTCHL);

	// read indicator data
	pDate->indicator = ((RegRead(R08_RTC_HOUR)) & RTCAP) >> RTCAP_SFT;

	// read day data
	pDate->day = RegRead(R08_RTC_DAY);

	// read month data
	pDate->month = RegRead(R08_RTC_MONTH);

	// read year data
	pDate->year = RegRead(R08_RTC_YEAR);

	// read week data
	pDate->week = RegRead(R08_RTC_WEEK);

	// disable RTC read buffer
	RegModify(R08_RTC_CNTL1, RTCRDHLD, (RTC_RBOFF << RTCRDHLD_SFT));

}

/**
 * @brief	RTC busy check.
 * - 1. set RTC counter hold.
 * - 2. check busy flag
 * - 3. wait a moment
 * - 4. set RTC counter running
 *
 * @return	none
 * @date	2010/04/01
 */
void RTC_checkBusy(void)
{
	while(1) {
		// set RTC counter hold register busy check
		RegModify(R08_RTC_CNTL1, RTCHLD, (RTC_COUNTER_HOLD << RTCHLD_SFT));

		// check RTC busy flag
		if (((RegRead(R08_RTC_CNTL1) & RTCBSY) >> RTCBSY_SFT) == RTC_POSSIBLE) {
			break;
		}

		// wait
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");

		// set RTC counter hold register normal run
		RegModify(R08_RTC_CNTL1, RTCHLD, (RTC_COUNTER_RUNNING << RTCHLD_SFT));
	}
}

/**
 * @brief	RTC interrupt initialize
 * - 1. check parameters.
 * - 2. set interrupt level
 * - 3. set interrupt cycle
 * - 4. set interrupt mode
 *
 * @param	intLevel;	unsigned char: 		interrupt level
 * -		RTC_INT_LEVEL0 : level0
 * -		RTC_INT_LEVEL1 : level1
 * -		RTC_INT_LEVEL2 : level2
 * -		RTC_INT_LEVEL3 : level3
 * -		RTC_INT_LEVEL4 : level4
 * -		RTC_INT_LEVEL5 : level5
 * -		RTC_INT_LEVEL6 : level6
 * -		RTC_INT_LEVEL7 : level7
 * @param	intCycle;	unsigned char: 		interrupt cycle
 * -		RTC_INT_CYCLE_1_64S : 1/64 second
 * -		RTC_INT_CYCLE_SECOND : 1 second
 * -		RTC_INT_CYCLE_MINUTE : 1 minute
 * -		RTC_INT_CYCLE_HOUR : 1 hour
 * -		RTC_INT_CYCLE_1_512S : 1/512 second
 * -		RTC_INT_CYCLE_1_256S : 1/256 second
 * -		RTC_INT_CYCLE_1_128S : 1/128 second
 * @param	intMode;	unsigned char: 		interrupt mode
 * -		RTC_INT_MODE_EDGE : edge trigger
 * -		RTC_INT_MODE_LEVEL : level sense
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_initInt(unsigned char intLevel, unsigned char intCycle, unsigned char intMode)
{
	// parameter check
	if(	(intLevel >= RTC_INT_LEVELMAX) ||
		(intCycle >= RTC_INT_CYCLE_MAX) ||
		(intMode > RTC_INT_MODE_LEVEL) ) {
		return RTC_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(RTC_INTLVL, intLevel);

	// set interrupt interval
	RegModify(R08_RTC_INTMODE, RTCT, (intCycle << RTCT_SFT));

	// set interrupt mode
	RegModify(R08_RTC_INTMODE, RTCIMD, (intMode << RTCIMD_SFT));

	return RTC_SUCCESS;
}

/**
 * @brief	RTC interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 * - 2. set RTC interrupt enable/disable.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		RTC_INT_DISABLE  : real time clock interrupt enable
 * -		RTC_INT_ENABLE : real time clock interrupt disable
 * @return	short:							end status
 * -		RTC_SUCCESS : success
 * -		RTC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short RTC_controlInt(unsigned char selectBit)
{
	// parameter check
	if (selectBit > RTC_INT_ENABLE) {
		return RTC_INVALID_PARAMETER;
	}

	// reset interrupt flag
	RTC_resetIntFlag();

	// set interrupt enable / disable
	RegModify(R08_RTC_INTMODE, RTCIEN, (selectBit << RTCIEN_SFT));

	return RTC_SUCCESS;
}

/**
 * @brief	RTC interrupt flag reset.
 * - 1. reset interrupt flag.
 *
 * @return	none
 * @date	2010/04/01
 */
void RTC_resetIntFlag(void)
{
	// reset interrupt flag
	RegWrite(R08_RTC_INSTAT, RTC_RESET);
}

/**
 * @brief	 RTC interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned char:			check interrupt flag
 * -		RTC_INT_NOTOCCURRED : RTC interrupt is not occurred
 * -		RTC_INT_OCCURRED    : RTC interrupt is occurred
 * @return	none
 * @date	2010/04/01
 */
void RTC_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_RTC_INSTAT);
}

#endif	// end of PE_RTC
