/**
 * @file	t16a2.c
 * @brief	PWM timer(T16A2) driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_T16A2
#include <t16a2_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static volatile unsigned char	*T16A2_INTLVL_REG[] = {
	T16A2_INTLVL_CH0,														// Ch. 0
	T16A2_INTLVL_CH1														// Ch. 1
};

/*
 * Declarations of function prototype
 */
short T16A2_init(unsigned char chNo, T16A2_INIT_PARAM *param);
short T16A2_setTimerMode(unsigned char chNo, T16A2_TIM_MODE *mode);
short T16A2_setComparatorCapture(unsigned char chNo, T16A2_CCCTL *cca, T16A2_CCCTL *ccb);
short T16A2_getCounterData(unsigned char chNo, unsigned short *data);
short T16A2_setCompareData(unsigned char chNo, unsigned short dataA, unsigned short dataB);
short T16A2_getCaptureData(unsigned char chNo, unsigned short *dataA, unsigned short *dataB);
short T16A2_resetTimer(unsigned char chNo);
short T16A2_setTimerRun(unsigned char chNo, unsigned short selectBit);
short T16A2_initInt(unsigned char chNo, unsigned char intLevel);
short T16A2_controlInt(unsigned char chNo, unsigned short selectBit);
short T16A2_resetIntFlag(unsigned char chNo, unsigned short selectBit);
short T16A2_checkIntFlag(unsigned char chNo, unsigned short *flag);


/**
 * @brief	PWM timer(T16A2) initialize.
 * - 1. check parameters.
 * - 2. stop a PWM timer.
 * - 3. set PWM timer ports.
 * - 4. set a timer mode.
 * - 5. set a compare data.(it is needed for a comparator mode.)
 * - 6. reset a counter.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*param; T16A2_INIT_PARAM: 		initialize parameter
 * -		cca; T16A2_CCCTL:				comparator/capture A control parameter
 * 	-			refer to T16A2_setComparatorCapture
 * -		ccb; T16A2_CCCTL:				comparator/capture B control parameter
 * 	-			refer to T16A2_setComparatorCapture
 * -		mode; T16A2_TIM_MODE:			timer mode parameter
 * 	-			refer to T16A2_setTimerMode
 * -		clock; unsigned char:			input clock source
 *  -			T16A2_CLKSRC_IOSC : IOSC
 *  -			T16A2_CLKSRC_OSC1 : OSC1
 *  -			T16A2_CLKSRC_OSC3 : OSC3
 *  -			T16A2_CLKSRC_EXCL : external clock
 * -		dataA; unsigned short:			compare data A
 * 	-			from 0x0000 to 0xFFFF
 * -		dataB; unsigned short:			compare data B
 * 	-			from 0x0000 to 0xFFFF
 * -		useEXCLS; unsigned char:		select external clock port
 * 	-			MUX_EXCLXS_OFF : not use external clock
 * 	-			MUX_EXCL5S_CH0_P36 : select P36 for external clock(Ch.0)
 * 	-			MUX_EXCL5S_CH0_P47 : select P47 for external clock(Ch.0)
 * 	-			MUX_EXCL6S_CH1_P37 : select P37 for external clock(Ch.1)
 * 	-			MUX_EXCL6S_CH1_P50 : select P50 for external clock(Ch.1)
 * -		useToutAPort; unsigned char:	select TOUTA port
 * 	-			MUX_TOUTAXP_OFF : not use TOUTA
 * 	-			MUX_TOUTA5P_CH0_P31 : select P31 for TOUTA5(Ch.0)
 * 	-			MUX_TOUTA5P_CH0_P51 : select P51 for TOUTA5(Ch.0)
 * 	-			MUX_TOUTA5P_CH0_P31_P51 : use both P31 and P51 for TOUTA5(Ch.0)
 * 	-			MUX_TOUTA6P_CH1_P33 : select P33 for TOUTA6(Ch.1)
 * 	-			MUX_TOUTA6P_CH1_P55 : select P55 for ROUTA6(Ch.1)
 * 	-			MUX_TOUTA6P_CH0_P33_P55 : use both P33 and P55 for TOUTA6(Ch.1)
 * -		useToutBPort; unsigned char:	select TOUTB port
 * 	-			MUX_TOUTBXP_OFF : not use TOUTB
 * 	-			MUX_TOUTB5P_CH0_P32 : select P32 for TOUTB5(Ch.0)
 * 	-			MUX_TOUTB5P_CH0_P52 : select P52 for TOUTB5(Ch.0)
 * 	-			MUX_TOUTB5P_CH0_P32_P52 : use both P32 and P52 for TOUTB5(Ch.0)
 * 	-			MUX_TOUTB6P_CH1_P34 : select P34 for TOUTB6(Ch.1)
 * 	-			MUX_TOUTB6P_CH1_P56 : select P56 for ROUTB6(Ch.1)
 * 	-			MUX_TOUTB6P_CH0_P34_P56 : use both P34 and P56 for TOUTB6(Ch.1)
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_init(unsigned char chNo, T16A2_INIT_PARAM *param)
{
	MUX_USE_T16A2PORT	use;											// port set parameter

	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		(param->mode.hcm > T16A2_HLFCLK_ENABLE) ||
		(param->mode.ccabcnt >= T16A2_CH_MAX) ||
		(param->mode.cbufen > T16A2_CMPBUF_ENABLE) ||
		(param->mode.trmd > T16A2_CNTMODE_ONESHOT) ||
		(param->cca.captrg >= T16A2_CAPTRG_MAX) ||
		(param->cca.toutmd >= T16A2_TOUT_MAX) ||
		(param->cca.toutinv > T16A2_TOUT_INVERT) ||
		(param->cca.ccmd > T16A2_MODE_CAPTURE) ||
		(param->ccb.captrg >= T16A2_CAPTRG_MAX) ||
		(param->ccb.toutmd >= T16A2_TOUT_MAX) ||
		(param->ccb.toutinv > T16A2_TOUT_INVERT) ||
		(param->ccb.ccmd > T16A2_MODE_CAPTURE) ||
#ifdef	S1C17624
		(param->useEXCLS >= MUX_EXCLXS_MAX) ||
		(param->useToutAPort >= MUX_TOUTAXP_MAX) ||
		(param->useToutBPort >= MUX_TOUTBXP_MAX) ||
#endif	// end of S1C17624
		(param->clock >= T16A2_CLKSRC_MAX) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// PWM timer stop
	T16A2_setTimerRun(chNo, T16A2_TIMER_STOP);

	// PWM timer port set
	if(param->clock == T16A2_CLKSRC_EXTERNAL) {							// select external clock
		use.useEXCL = MUX_EXCL_ON;
	} else {
		use.useEXCL = MUX_EXCL_OFF;
	}
	if(	(param->cca.ccmd == T16A2_MODE_CAPTURE)  ||
		(param->cca.toutmd != T16A2_TOUT_OFF) ) {						// select comparator/capture/TOUT mode
			use.useToutA = MUX_TOUTA_ON;
	} else {
			use.useToutA = MUX_TOUTA_OFF;
	}
	if(	(param->ccb.ccmd == T16A2_MODE_CAPTURE) ||
		(param->ccb.toutmd != T16A2_TOUT_OFF) ) {						// select comparator/capture/TOUT mode
			use.useToutB = MUX_TOUTB_ON;
	} else {
			use.useToutB = MUX_TOUTB_OFF;
	}

#ifdef S1C17624
	use.useEXCLS = param->useEXCLS;						// select EXCLS port
	use.useToutAPort = param->useToutAPort;				// select TOUTA port
	use.useToutBPort = param->useToutBPort;				// select TOUTB port
#endif // end of S1C17624

	MUX_setT16A2port(chNo, &use, MUX_ON);

	// PWM timer mode set
	T16A2_setTimerMode(chNo, &param->mode);
	T16A2_setComparatorCapture(chNo, &param->cca, &param->ccb);

	// compare data set
	if(	(param->cca.ccmd == T16A2_MODE_COMPARATOR) ||
		(param->ccb.ccmd == T16A2_MODE_COMPARATOR) ) {
			T16A2_setCompareData(chNo, param->dataA, param->dataB);
	}

	// PWM timer reset
	T16A2_resetTimer(chNo);

	return	T16A2_SUCCESS;
}

/**
 * @brief	set an PWM timer(T16A2) mode.
 * - 1. check parameters.
 * - 2. set an PWM timer mode.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*mode; T16A2_TIM_MODE:			mode set parameter
 * -		ccabcnt; unsigned short:		select connect counter
 * 	-			T16A2_CH0 : channel 0
 * 	-			T16A2_CH1 : channel 1
 * -		cbufen; unsigned short:			select compare data buffer write enable / disable
 * 	-			T16A2_CMPBUF_DISABLE : write disable
 * 	-			T16A2_CMPBUF_ENABLE  : write enable
 * -		trmd; unsigned short:			select count mode
 * 	-			T16A2_CNTMODE_REPEAT  : repeat mode
 * 	-			T16A2_CNTMODE_ONESHOT : one shot mode
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_setTimerMode(unsigned char chNo, T16A2_TIM_MODE *mode)
{
	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		(mode->hcm > T16A2_HLFCLK_ENABLE) ||
		(mode->ccabcnt >= T16A2_CH_MAX) ||
		(mode->cbufen > T16A2_CMPBUF_ENABLE) ||
		(mode->trmd > T16A2_CNTMODE_ONESHOT) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// PWM timer mode set
	RegModify(R16_T16A_CTL(chNo), (HCM | CCABCNT | R16_T16A_CTL_CBUFEN | T16ATRMD),
		((mode->hcm << HCM_SFT) | (mode->ccabcnt << CCABCNT_SFT) | (mode->cbufen << R16_T16A_CTL_CBUFEN_SFT) | (mode->trmd << T16ATRMD_SFT)));

	return	T16A2_SUCCESS;
}

/**
 * @brief	set a comparator / capture mode.
 * - 1. check parameters.
 * - 2. set a comparator/capture of an PWM timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*cca; T16A2_CCCTL:				comparator/capture A set parameter
 * -		captrg; unsigned short:			select trigger edge of a external signal
 * 	-			T16A2_CAPTRG_NONE : trigger none
 * 	-			T16A2_CAPTRG_R    : rising edge
 * 	-			T16A2_CAPTRG_F    : falling edge
 * 	-			T16A2_CAPTRG_RF   : falling and rising edge
 * -		toutmd; unsigned short:			select mode of a TOUT signal
 * 	-			T16A2_TOUT_OFF   : output off
 * 	-			T16A2_TOUT_AR_BF : compare A rising, compare B falling
 * 	-			T16A2_TOUT_ARF   : compare A toggle
 * 	-			T16A2_TOUT_BRF   : compare B toggle
 * -		toutinv; unsigned short:		select TOUT signal polarity
 * 	-			T16A2_TOUT_NORMAL : TOUT signal normal
 * 	-			T16A2_TOUT_INVERT : TOUT signal invert
 * -		ccmd; unsigned short:			select comparator/capture mode
 * 	-			T16A2_MODE_COMPARATOR : comparator mode
 * 	-			T16A2_MODE_CAPTURE    : capture mode
 * @param	*ccb; T16A2_CCCTL:				comparator/capture B set parameter
 * -		refer to parameter of *cca
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_setComparatorCapture(unsigned char chNo, T16A2_CCCTL *cca, T16A2_CCCTL *ccb)
{
	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		(cca->captrg >= T16A2_CAPTRG_MAX) ||
		(cca->toutmd >= T16A2_TOUT_MAX) ||
		(cca->toutinv > T16A2_TOUT_INVERT) ||
		(cca->ccmd > T16A2_MODE_CAPTURE) ||
		(ccb->captrg >= T16A2_CAPTRG_MAX) ||
		(ccb->toutmd >= T16A2_TOUT_MAX) ||
		(ccb->toutinv > T16A2_TOUT_INVERT) ||
		(ccb->ccmd > T16A2_MODE_CAPTURE)	) {
			return	T16A2_INVALID_PARAMETER;
	}

	// comparator/capture mode set
	RegWrite(R16_T16A_CCCTL(chNo), ((ccb->captrg << CAPBTRG_SFT) | (ccb->toutmd << TOUTBMD_SFT) |
		(ccb->toutinv << TOUTBINV_SFT) | (ccb->ccmd << CCBMD_SFT) | (cca->captrg << CAPATRG_SFT) |
		(cca->toutmd << TOUTAMD_SFT) | (cca->toutinv << TOUTAINV_SFT) | (cca->ccmd << CCAMD_SFT)));

	return	T16A2_SUCCESS;
}

/**
 * @brief	get a counter data.
 * - 1. check parameters.
 * - 2. get a counter data from register and set the data to "data".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*data; unsigned short:			counter data
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_getCounterData(unsigned char chNo, unsigned short *data)
{
	// parameter check
	if(chNo >= T16A2_CH_MAX) {
		return	T16A2_INVALID_PARAMETER;
	}

	// counter data get
	*data = RegRead(R16_T16A_TC(chNo));

	return	T16A2_SUCCESS;
}

/**
 * @brief	set an PWM timer compare data A/B
 * - 1. check parameters.
 * - 2. set a compare data A.
 * - 3. set a compare data B.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	dataA; unsigned short: 			compare data A
 * -		from 0x0000 to 0xFFFF
 * @param	dataB; unsigned short: 			compare data B
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_setCompareData(unsigned char chNo, unsigned short dataA, unsigned short dataB)
{
	// parameter check
	if(chNo >= T16A2_CH_MAX) {
		return	T16A2_INVALID_PARAMETER;
	}

	// compare data set
	RegWrite(R16_T16A_CCA(chNo), dataA);
	RegWrite(R16_T16A_CCB(chNo), dataB);

	return	T16A2_SUCCESS;
}

/**
 * @brief	get a capture data.
 * - 1. check parameters.
 * - 2. get a capture data A from register and set the data to "dataA".
 * - 3. get a capture data B from register and set the data to "dataB".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*dataA; unsigned short:			capture A data
 * -		from 0x0000 to 0xFFFF
 * @param	*dataB; unsigned short:			capture B data
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_getCaptureData(unsigned char chNo, unsigned short *dataA, unsigned short *dataB)
{
	// parameter check
	if(chNo >= T16A2_CH_MAX) {
		return	T16A2_INVALID_PARAMETER;
	}

	// capture data get
	*dataA = RegRead(R16_T16A_CCA(chNo));
	*dataB = RegRead(R16_T16A_CCB(chNo));

	return	T16A2_SUCCESS;
}

/**
 * @brief	reset a PWM timer.
 * - 1. check parameters.
 * - 2. reset an PWM timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_resetTimer(unsigned char chNo)
{
	// parameter check
	if(chNo >= T16A2_CH_MAX) {
		return	T16A2_INVALID_PARAMETER;
	}

	// PWM timer reset
	RegModify(R16_T16A_CTL(chNo), T16ARST, (T16ARST * T16A_RESET));

	return	T16A2_SUCCESS;
}

/**
 * @brief	set an PWM timer run / stop.
 * - 1. check parameters.
 * - 2. set an PWM timer run/stop.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	selectBit; unsigned short: 		select run / stop
 * -		T16A2_TIMER_STOP : PWM timer stop
 * -		T16A2_TIMER_RUN  : PWM timer run
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_setTimerRun(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		(selectBit > T16A2_TIMER_RUN) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// PWM timer run / stop
	RegModify(R16_T16A_CTL(chNo), T16ARUN, (selectBit << T16ARUN_SFT));

	return	T16A2_SUCCESS;
}

/**
 * @brief	PWM timer(T16A2) interrupt initialize.
 * - 1. check parameters.
 * - 2. set a compare A/B, capture A/B interrupt level.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	intLevel; unsigned char:		select compare A/B, capture A/B interrupt level
 * -		T16A2_INT_LEVEL0 : level0
 * -		T16A2_INT_LEVEL1 : level1
 * -		T16A2_INT_LEVEL2 : level2
 * -		T16A2_INT_LEVEL3 : level3
 * -		T16A2_INT_LEVEL4 : level4
 * -		T16A2_INT_LEVEL5 : level5
 * -		T16A2_INT_LEVEL6 : level6
 * -		T16A2_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_initInt(unsigned char chNo, unsigned char intLevel)
{
	// parameter check
	if( (chNo >= T16A2_CH_MAX) ||
		(intLevel >= T16A2_INT_LEVELMAX) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(T16A2_INTLVL_REG[chNo], intLevel);

	return	T16A2_SUCCESS;
}

/**
 * @brief	PWM timer(T16A2) interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set a compare A/B, capture A/B interrupt enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	selectBit; unsigned short:		select compare A/B, capture A/B interrupt enable / disable
 * -		T16A2_CAPBOVW_DISABLE : capture B overwrite interrupt disable
 * -		T16A2_CAPBOVW_ENABLE  : capture B overwrite interrupt enable
 * -		T16A2_CAPAOVW_DISABLE : capture A overwrite interrupt disable
 * -		T16A2_CAPAOVW_ENABLE  : capture A overwrite interrupt enable
 * -		T16A2_CAPBINT_DISABLE : capture B interrupt disable
 * -		T16A2_CAPBINT_ENABLE  : capture B interrupt enable
 * -		T16A2_CAPAINT_DISABLE : capture A interrupt disable
 * -		T16A2_CAPAINT_ENABLE  : capture A interrupt enable
 * -		T16A2_CMPBINT_DISABLE : compare B interrupt disable
 * -		T16A2_CMPBINT_ENABLE  : compare B interrupt enable
 * -		T16A2_CMPAINT_DISABLE : compare A interrupt disable
 * -		T16A2_CMPAINT_ENABLE  : compare A interrupt enable
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_controlInt(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		((selectBit & ~T16A_INT_MASK) != 0) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// reset interrupt flag
	T16A2_resetIntFlag(chNo, selectBit);

	// set interrupt enable
	RegModify(R16_T16A_IEN(chNo), T16A_INT_MASK, selectBit);

	return	T16A2_SUCCESS;
}

/**
 * @brief	PWM timer(T16A2) interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	selectBit; unsigned short:		reset interrupt status
 * -		T16A2_CAPBOVW_RESET : capture B overwrite interrupt status reset
 * -		T16A2_CAPAOVW_RESET : capture A overwrite interrupt status reset
 * -		T16A2_CAPBINT_RESET : capture B interrupt status reset
 * -		T16A2_CAPAINT_RESET : capture A interrupt status reset
 * -		T16A2_CMPBINT_RESET : compare B interrupt status reset
 * -		T16A2_CMPAINT_RESET : compare A interrupt status reset
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_resetIntFlag(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16A2_CH_MAX) ||
		((selectBit & ~T16A_INT_MASK) != 0) ) {
			return	T16A2_INVALID_PARAMETER;
	}

	// reset interrupt flag
	RegWrite(R16_T16A_IFLG(chNo), selectBit);

	return	T16A2_SUCCESS;
}

/**
 * @brief	PWM timer(T16A2) interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16A2_CH0 : channel 0
 * -		T16A2_CH1 : channel 1
 * @param	*flag; unsigned short:			check interrupt flag
 * -		T16A2_CAPBOVW_NOTOCCURRED : capture B overwrite interrupt is not occurred
 * -		T16A2_CAPBOVW_OCCURRED    : capture B overwrite interrupt is occurred
 * -		T16A2_CAPAOVW_NOTOCCURRED : capture A overwrite interrupt is not occurred
 * -		T16A2_CAPAOVW_OCCURRED    : capture A overwrite interrupt is occurred
 * -		T16A2_CAPBINT_NOTOCCURRED : capture B interrupt is not occurred
 * -		T16A2_CAPBINT_OCCURRED    : capture B interrupt is occurred
 * -		T16A2_CAPAINT_NOTOCCURRED : capture A interrupt is not occurred
 * -		T16A2_CAPAINT_OCCURRED    : capture A interrupt is occurred
 * -		T16A2_CMPBINT_NOTOCCURRED : compare B interrupt is not occurred
 * -		T16A2_CMPBINT_OCCURRED    : compare B interrupt is occurred
 * -		T16A2_CMPAINT_NOTOCCURRED : compare A interrupt is not occurred
 * -		T16A2_CMPAINT_OCCURRED    : compare A interrupt is occurred
 * @return	short:							end status
 * -		T16A2_SUCCESS : success
 * -		T16A2_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/01
 */
short T16A2_checkIntFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= T16A2_CH_MAX) {
		return	T16A2_INVALID_PARAMETER;
	}

	// get interrupt flag
	*flag = (RegRead(R16_T16A_IFLG(chNo)) & T16A_INT_MASK) >> CAIF_SFT;

	return	T16A2_SUCCESS;
}

#endif	// end of PE_T16A2


