/**
 * @file	wdt.c
 * @brief	watchdog timer driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_WDT
#include <wdt_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
void WDT_resetTimer(void);
void WDT_setTimerRun(unsigned char selectBit);
short WDT_setTimerMode(unsigned char mode);
short WDT_checkNMI(void);


/**
 * @brief	reset a watchdog timer.
 * - 1. reset a watchdog timer.
 *
 * @return	none
 * @date	2009/12/25
 */
void WDT_resetTimer(void)
{
	// watchdog timer reset
	RegModify(R08_WDT_CTL, WDTRST, (WDTRST * WDT_RESET));
}

/**
 * @brief	set a watchdog timer run / stop.
 * - 1. reset a watchdog timer.
 * - 2. set a watchdog timer run/stop.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		WDT_TIMER_STOP : watchdog timer stop
 * -		WDT_TIMER_RUN  : watchdog timer run
 * @return	none
 * @date	2009/12/25
 */
void WDT_setTimerRun(unsigned char selectBit)
{
	// watchdog timer reset
	WDT_resetTimer();

	// watchdog timer run / stop
	RegModify(R08_WDT_CTL, WDTRUN, (selectBit << WDTRUN_SFT));
}

/**
 * @brief	set a watchdog timer mode.
 * - 1. check parameters.
 * - 2. set a watchdog timer mode.
 *
 * @param	mode; unsigned char: 			select mode
 * -		WDT_TIMMODE_NMI   : NMI mode
 * -		WDT_TIMMODE_RESET : RESET mode
 * @return	short:							end status
 * -		WDT_SUCCESS : success
 * -		WDT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short WDT_setTimerMode(unsigned char mode)
{
	// parameter check
	if(mode > WDT_RESET) {
		return	WDT_INVALID_PARAMETER;
	}

	// watchdog timer mode set
	RegModify(R08_WDT_ST, WDTMD, (mode << WDTMD_SFT));

	return	WDT_SUCCESS;
}

/**
 * @brief	check a NMI interrupt.
 * - 1. read an interrupt status and set the status to return value.
 *
 * @return	short:							end status
 * -		WDT_NOTOCCURRED : NMI is not occurred
 * -		WDT_OCCURRED    : NMI is occurred(counter over flow)
 * @date	2010/04/15 : Add cast to return short value.
 * @version
 * -		2009/12/25
 */
short WDT_checkNMI(void)
{
	// get NMI Status
	return	(short)(RegRead(R08_WDT_ST) & WDTST);
}

#endif	// end of PE_WDT
